@@ -4,9 +4,9 @@ from secrets import token_hex
 from typing import Any, Dict
 
 import logging_loki
-from chaoslib import __version__
+from chaoslib import __version__, experiment_hash
 from chaoslib.run import EventHandlerRegistry, RunEventHandler
-from chaoslib.types import Experiment, Journal, Secrets
+from chaoslib.types import Activity, Experiment, Journal, Run, Secrets
 from logzero import logger as ctk_logger
 
 __all__ = ["configure_control"]
@@ -20,7 +20,21 @@ def configure_control(
     event_registry: EventHandlerRegistry = None,
     loki_endpoint: str = DEFAULT_LOKI_URL,
     tags: Dict[str, str] = None,
+    experiment_ref: str = None,
+    trace_id: str = None,
 ) -> None:
+    """
+    Configure a Python logger that sends its messages to a Loki endpoint.
+
+    * `loki_endpoint` is teh base url of your Loki service
+    * `tags` a mapping of strings injected in all logs
+    * `experiment_ref` a unique string identifying this experiment, if none
+      is provided, a has of the experiment is created
+    * `trace_id` a unique string for a particular run of the experiment, if
+      none is provided, a random string is generated
+
+    This sends logs about the run events (started, finished, failed, etc.)
+    """
     ctk_logger.debug("Add Loki handler to logger")
 
     if event_registry is None:
@@ -33,17 +47,22 @@ def configure_control(
 
     url = f"{loki_endpoint}/loki/api/v1/push"
     auth = secrets.get("auth")
-    title = experiment.get("title")
+    experiment_ref = experiment_ref or experiment_hash(experiment)
+    trace_id = trace_id or token_hex(16)
     tags = tags or {}
     tags.update(
         {
             "source": "chaostoolkit",
             "chaostoolkit_lib_version": __version__,
-            "chaostoolkit_run_id": token_hex(16),
-            "chaostoolkit_experiment_title": title,
+            "chaostoolkit_run_trace_id": trace_id,
+            "chaostoolkit_experiment_ref": experiment_ref,
         }
     )
 
+    experiment_tags = experiment.get("tags")
+    if experiment_tags:
+        tags.update(experiment_tags)
+
     logging_loki.emitter.LokiEmitter.level_tag = "level"
     handler = logging_loki.LokiHandler(
         url=url, tags=tags, auth=auth, version="1"
@@ -56,7 +75,44 @@ def configure_control(
     event_registry.register(LokiRunEventHandler())
 
     loki_logger.info(
-        "Experiment started", extra={"tags": {"type": "experiment-started"}}
+        "Experiment started",
+        extra={
+            "tags": {
+                "type": "experiment-started",
+                "title": experiment.get("title"),
+            }
+        },
+    )
+
+
+def before_activity_control(context: Activity, *args, **kwargs) -> None:
+    a = context
+    loki_logger.info(
+        f"Activity '{a['name']}' started",
+        extra={
+            "tags": {"type": "experiment-activity-started", "name": a["name"]},
+        },
+    )
+
+
+def after_activity_control(
+    context: Activity, state: Run, *args, **kwargs
+) -> None:
+    a = context
+    loki_logger.info(
+        f"Activity '{a['name']}' finished",
+        extra={
+            "tags": {
+                "type": "experiment-activity-finished",
+                "name": a["name"],
+                "status": state["status"],
+                "start": state["start"],
+                "end": state["end"],
+                "duration": state["duration"],
+                "output": state["output"],
+                "exception": state.get("exception"),
+            },
+        },
     )
 
 
@@ -79,7 +135,16 @@ class LokiRunEventHandler(RunEventHandler):
     def finish(self, journal: Journal) -> None:
         loki_logger.info(
             "Experiment finished",
-            extra={"tags": {"type": "experiment-finished"}},
+            extra={
+                "tags": {
+                    "type": "experiment-finished",
+                    "deviated": journal.get("deviated"),
+                    "status": journal.get("status"),
+                    "start": journal.get("start"),
+                    "end": journal.get("end"),
+                    "duration": journal.get("duration"),
+                },
+            },
         )
 
     def interrupted(self, experiment: Experiment, journal: Journal) -> None:
@@ -133,9 +198,21 @@ class LokiRunEventHandler(RunEventHandler):
     def hypothesis_before_completed(
         self, experiment: Experiment, state: Dict[str, Any], journal: Journal
     ) -> None:
+        extra = {
+            "tags": {
+                "type": "experiment-before-ssh-completed",
+                "state_met": state.get("steady_state_met"),
+            },
+        }
+
+        if state.get("steady_state_met") is False:
+            for probe in state.get("probes", []):
+                if probe["tolerance_met"] is False:
+                    extra["tags"]["failed_probe"] = probe
+                    break
+
         loki_logger.info(
-            "Experiment before steady state completed",
-            extra={"tags": {"type": "experiment-before-ssh-completed"}},
+            "Experiment before steady state completed", extra=extra
         )
 
     def start_hypothesis_after(self, experiment: Experiment) -> None:
@@ -147,10 +224,20 @@ class LokiRunEventHandler(RunEventHandler):
     def hypothesis_after_completed(
         self, experiment: Experiment, state: Dict[str, Any], journal: Journal
     ) -> None:
-        loki_logger.info(
-            "Experiment after steady state completed",
-            extra={"tags": {"type": "experiment-after-ssh-completed"}},
-        )
+        extra = {
+            "tags": {
+                "type": "experiment-after-ssh-completed",
+                "state_met": state.get("steady_state_met"),
+            },
+        }
+
+        if state.get("steady_state_met") is False:
+            for probe in state.get("probes", []):
+                if probe["tolerance_met"] is False:
+                    extra["tags"]["failed_probe"] = probe
+                    break
+
+        loki_logger.info("Experiment after steady state completed", extra=extra)
 
     def start_method(self, experiment: Experiment) -> None:
         loki_logger.info(
